/*
 * File:   main.c
 *
 * Boombox Transmission Module
 *
 * Cheng Xing, Rene Zhang
 * cx34, rz99
 * December 2015
 */

// === Config ==================================================================
#include "config.h"

// === Libraries ===============================================================
// Standard libraries
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

// Ring Buffer Library
#include "ring_buf.h"

// Audio Hardware Library
#include "adc.h"

// Reception Library
#include "audio_receive.h"

// Compression Library
#include "adpcm.h"

// === Audio Ring Buffers ======================================================
#define RING_SIZE 4096
// Circular ring buffer for audio
volatile static ring_buf_t ringL;
volatile static ring_buf_t ringR;

// === SPI 1 Interrupt Handler =================================================
/*
 * Interrupt handler for SPI 1.
 * DAC write is triggerd on timer interrupt
 */

// DAC ISR variables
volatile int DAC_LR = 0;
volatile int readL = 0;
volatile int readR = 0;

// ADPCM Decoding State Machines
signed long prev_sampleL = 0;
int         prev_indexL  = 0;
signed long prev_sampleR = 0;
int         prev_indexR  = 0;

void __ISR(_SPI_1_VECTOR, ipl4) SPI1InterruptHandler(void) {
  if (DAC_LR) {
    if (!ring_empty(ringL)) {
      readL = ring_pop(ringL);
      readL = ADPCMDecoder(readL, &prev_sampleL, &prev_indexL);
    }
    dac_write(readL);
  } else {
    if (!ring_empty(ringR)) {
      readR = ring_pop(ringR);
      readR = ADPCMDecoder(readR, &prev_sampleR, &prev_indexR);
    }
    dac_write(readR);
  }
  DAC_LR = 1 - DAC_LR;

  // Clear interrupt
  INTClearFlag(INT_SPI1TX);
}

// === Main  ===================================================================

void main() {
  // === Setup chip ============================================================
  SYSTEMConfig(sys_clock, SYS_CFG_WAIT_STATES | SYS_CFG_PCACHE);

  ANSELA =0; //make sure analog is cleared
  ANSELB =0;

  int i;

  // === Setup input ring buffer ===============================================
  ringL = ring_new(RING_SIZE);
  ringR = ring_new(RING_SIZE);

  // === Setup DAC =============================================================
  dac_setup();

  // === Setup Audio Data Receive ==============================================
  ar_setup();
  unsigned char left_buf[AR_CHN_PKT_SIZE];
  unsigned char right_buf[AR_CHN_PKT_SIZE];

  // === setup system wide interrupts  =========================================
  INTEnableSystemMultiVectoredInt();

  // === Blinking debug LED ====================================================
  mPORTASetPinsDigitalOut(BIT_0);
  mPORTASetBits(BIT_0);
  int blink = 0;

  // === Reception loop ========================================================
  while (1) {
    // Receive audio data
    ar_receive(left_buf, right_buf);

    // Push received data into ring buffer
    for (i = 0; i < AR_CHN_PKT_SIZE; i++) {
      ring_push(ringL, left_buf[i] );
      ring_push(ringR, right_buf[i]);
    }

    // Block while the number of unread packets is too high
    while (ring_size(ringL) > RING_SIZE / 2 || ring_size(ringR) > RING_SIZE / 2);

    // Toggle debug LED
    if (blink++ % 20 == 0) {
      mPORTAToggleBits(BIT_0);
    }
  }

}